<?php

namespace Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use \Utilities\MessagesProcedure;
use Zeedhi\Framework\DTO\Response\Error;
use Doctrine\DBAL\Connection;
// use Zeedhi\Framework\DB\StoredProcedure\Param;
// use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class ParametrosCompanhia extends \Teknisa\GER\Controllers\TeknisaCrud {

	protected $dataSourceName;
    protected $parameterBagColumns = array();

    public function __construct($crudService, $parameterBag,  $environment, $dataSourceManager) {
        $this->crudService = $crudService;
        $this->parameterBag=$parameterBag;
        $this->environment=$environment;
        $this->dataSourceManager = $dataSourceManager;
    }

    private function validateIntervalDate($row, $dataSourceName, $response, $dataIni, $dataFim){
        $this->dataSourceName = $dataSourceName;
        $this->parameterBagColumns = array();
        $isValid = true;
        $condition = array(
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $row["CDFILIAL"]
            ),
            array(
                "columnName" => "CDCIA",
                "operator" => "=",
                "value" => $row["CDCIA"]
            )
        );
        if(!$row["__is_new"] && array_key_exists("CDPURCHASEORDER", $row)){
            array_push($condition, array(
                "columnName" => "CDPURCHASEORDER",
                "operator" => "<>",
                "value" => $row["CDPURCHASEORDER"]
            ));
        }
        $filterCriteria= new FilterCriteria($this->dataSourceName, $condition);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        $registers = $this->dataSourceManager->findBy($newFilterCriteria);
        $rows = $registers->getRows();
        if(!empty($rows)){
            foreach ($rows as $record) {
                if(!($this->returnStrToTime($row[$dataIni]) > $this->returnStrToTime($record[$dataFim]) ||
                   $this->returnStrToTime($row[$dataFim]) < $this->returnStrToTime($record[$dataIni]))){
                    $isValid = false;
                    break;
                }
            }
        }
        return $isValid;        
    }

    public function saveDesconto(DTO\Request\Row $request, DTO\Response $response){
        $this->dataSourceName = 'despercia';
        $row = $request->getRow();
        $validate = $this->validateIntervalDate($row, $this->dataSourceName, $response, "DTINIDESC", "DTFIMDESC");
        if($validate){
            parent::save($request, $response);
        } else {
            $response->setError(new Error("Impossível adicionar. Já existem Descontos para o período informado.", 500));
        }
    }
    
    public function savePurchaseOrder(DTO\Request\Row $request, DTO\Response $response){
        $this->dataSourceName = 'purchaseorder';
        $row = $request->getRow();
        $validate = $this->validateIntervalDate($row, $this->dataSourceName, $response, "DTINIVIGENCIA", "DTFIMVIGENCIA");
        if($validate){
            parent::save($request, $response);
        } else {
            $response->setError(new Error("Impossível adicionar. Existem Purchase Order cadastrados para o período informado.", 500));
        }
    }

    private function returnStrToTime($data){
        $data = explode(' ', $data)[0];
        $date = \DateTime::createFromFormat('d/m/Y', $data, new \DateTimeZone('America/Sao_Paulo'));
        return $date->format('Y-m-d');
    }

}